@@ -1,6 +1,6 @@
 /* ncdu - NCurses Disk Usage 
   
-  Copyright (c) 2007 Yoran Heling
+  Copyright (c) 2007-2008 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -29,31 +29,26 @@ struct dir *bcur;
 
 
 void drawInfo(struct dir *dr) {
-  WINDOW *nfo;
   char path[PATH_MAX];
 
-  nfo = newwin(11, 60, winrows/2-5, wincols/2-30);
-  box(nfo, 0, 0);
-  wattron(nfo, A_BOLD);
-  mvwaddstr(nfo, 0, 4, "Item info");
+  nccreate(11, 60, "Item info");
 
-  mvwaddstr(nfo, 2, 3, "Name:");
-  mvwaddstr(nfo, 3, 3, "Path:");
-  mvwaddstr(nfo, 4, 3, "Type:");
-  mvwaddstr(nfo, 6, 3, "   Disk usage:");
-  mvwaddstr(nfo, 7, 3, "Apparent size:");
-  wattroff(nfo, A_BOLD);
+  attron(A_BOLD);
+  ncaddstr(2, 3, "Name:");
+  ncaddstr(3, 3, "Path:");
+  ncaddstr(4, 3, "Type:");
+  ncaddstr(6, 3, "   Disk usage:");
+  ncaddstr(7, 3, "Apparent size:");
+  attroff(A_BOLD);
 
-  mvwaddstr(nfo, 2,  9, cropdir(dr->name, 49));
-  mvwaddstr(nfo, 3,  9, cropdir(getpath(dr, path), 49));
-  mvwaddstr(nfo, 4,  9, dr->flags & FF_DIR ? "Directory"
+  ncaddstr(2,  9, cropdir(dr->name, 49));
+  ncaddstr(3,  9, cropdir(getpath(dr, path), 49));
+  ncaddstr(4,  9, dr->flags & FF_DIR ? "Directory"
       : dr->flags & FF_FILE ? "File" : "Other (link, device, socket, ..)");
-  mvwprintw(nfo, 6, 18, "%s (%s B)", cropsize(dr->size),  fullsize(dr->size));
-  mvwprintw(nfo, 7, 18, "%s (%s B)", cropsize(dr->asize), fullsize(dr->asize));
+  ncprint(6, 18, "%s (%s B)", cropsize(dr->size),  fullsize(dr->size));
+  ncprint(7, 18, "%s (%s B)", cropsize(dr->asize), fullsize(dr->asize));
 
-  mvwaddstr(nfo, 9, 32, "Press any key to continue");
-  wrefresh(nfo);
-  delwin(nfo);
+  ncaddstr(9, 32, "Press any key to continue");
 }
 
 
@@ -61,11 +56,6 @@ int cmp(struct dir *x, struct dir *y) {
   struct dir *a, *b;
   int r = 0;
 
-  if(y->flags & FF_PAR)
-    return(1);
-  if(x->flags & FF_PAR)
-    return(-1);
-
   if(bflags & BF_DESC) {
     a = y; b = x;
   } else {
@@ -145,12 +135,12 @@ char *graph(off_t max, off_t size) {
 
 
 #define exlhid(x) if(bflags & BF_HIDE && (\
-    (!(x->flags & FF_PAR) && x->name[0] == '.')\
+    (x != &ref && (x->name[0] == '.' || x->name[strlen(x->name)-1] == '~'))\
     || x->flags & FF_EXL)\
   ) { i--; continue; }
 
 void drawBrowser(int change) {
-  struct dir *n;
+  struct dir *n, ref;
   char tmp[PATH_MAX], ct, dt, *size;
   int selected, i, o;
   off_t max = 1;
@@ -175,12 +165,16 @@ void drawBrowser(int change) {
   mvhline(winrows-1, 0, ' ', wincols);
   mvprintw(0,0,"%s %s ~ Use the arrow keys to navigate, press ? for help", PACKAGE_NAME, PACKAGE_VERSION);
 
+  strcpy(tmp, cropsize(bcur->parent->size));
   mvprintw(winrows-1, 0, " Total disk usage: %s  Apparent size: %s  Items: %d",
-    cropsize(bcur->parent->size), cropsize(bcur->parent->asize), bcur->parent->items);
+    tmp, cropsize(bcur->parent->asize), bcur->parent->items);
   attroff(A_REVERSE);
   
   mvhline(1, 0, '-', wincols);
-  mvaddstr(1, 3, cropdir(getpath(bcur, tmp), wincols-5));
+  mvaddch(1, 3, ' ');
+  getpath(bcur, tmp);
+  mvaddstr(1, 4, cropdir(tmp, wincols-8));
+  mvaddch(1, 4+(strlen(tmp) > wincols-8 ? wincols-8 : strlen(tmp)), ' ');
 
  /* make sure the items are in correct order */
   if(!(bflags & BF_SORT)) {
@@ -189,6 +183,15 @@ void drawBrowser(int change) {
     bflags |= BF_SORT;
   }
 
+ /* add reference to parent dir */
+  memset(&ref, 0, sizeof(struct dir));
+  if(bcur->parent->parent) {
+    ref.name = "..";
+    ref.next = bcur;
+    ref.parent = bcur->parent;
+    bcur = &ref;
+  }
+
  /* get maximum size and selected item */
   for(n = bcur, selected = i = 0; n != NULL; n = n->next, i++) {
     exlhid(n)
@@ -241,7 +244,7 @@ void drawBrowser(int change) {
       attron(A_REVERSE);
 
    /* reference to parent dir has a different format */
-    if(n->flags & FF_PAR) {
+    if(n == &ref) {
       mvhline(i+2, 0, ' ', wincols);
       o = bgraph == 0 ? 11 :
           bgraph == 1 ? 23 :
@@ -295,6 +298,13 @@ void drawBrowser(int change) {
     if(i == selected)
       attroff(A_REVERSE);
   }
+
+ /* move cursor to selected row for accessibility */
+  move(selected+2, 0);
+
+ /* remove reference to parent dir */
+  if(bcur == &ref)
+    bcur = ref.next;
 }
 
 struct dir * selected(void) {
@@ -303,6 +313,8 @@ struct dir * selected(void) {
     if(n->flags & FF_BSEL)
       return n;
   } while((n = n->next) != NULL);
+  if(bcur->parent->parent)
+    return(bcur->parent);
   return NULL;
 }
 
@@ -380,7 +392,7 @@ void showBrowser(void) {
       case 10:
       case KEY_RIGHT:
         n = selected();
-        if(n->flags & FF_PAR)
+        if(n == bcur->parent)
           bcur = bcur->parent->parent->sub;
         else if(n->sub != NULL)
           bcur = n->sub;
@@ -416,17 +428,6 @@ void showBrowser(void) {
             t->items += bcur->items;
           }
 
-         /* add reference to parent dir */
-          if(bcur->parent) {
-            t = calloc(sizeof(struct dir), 1);
-            t->name = malloc(3);
-            t->flags |= FF_PAR;
-            strcpy(t->name, "..");
-            t->parent = bcur;
-            t->next = bcur->sub;
-            bcur->sub = t;
-          }
-
           bcur = bcur->sub;
           free(n->name);
           free(n);
@@ -445,7 +446,7 @@ void showBrowser(void) {
         break;
       case 'i':
         n = selected();
-        if(!(n->flags & FF_PAR)) {
+        if(n != bcur->parent) {
           drawInfo(n);
           while(getch() == KEY_RESIZE) {
             drawBrowser(0);
@@ -455,7 +456,7 @@ void showBrowser(void) {
         break;
       case 'd':
         n = selected();
-        if(!(n->flags & FF_PAR))
+        if(n != bcur->parent)
           bcur = showDelete(n);
         if(bcur && bcur->parent)
           bcur = bcur->parent->sub;
