@@ -1,6 +1,6 @@
 /* ncdu - NCurses Disk Usage 
    
-  Copyright (c) 2007 Yoran Heling
+  Copyright (c) 2007-2008 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -60,6 +60,8 @@ char *rpath(const char *from, char *to) {
       strcpy(tmp, cwd);
     else
       tmp[0] = 0;
+    if(strlen(cur) + 2 > PATH_MAX - strlen(tmp))
+      return(NULL);
     strcat(tmp, "/");
     strcat(tmp, cur);
   } else
@@ -135,28 +137,23 @@ char *rpath(const char *from, char *to) {
 
 /* the progress window */
 static void drawProgress(char *cdir) {
-  WINDOW *prg;
   char ani[15];
   int i;
 
-  prg = newwin(10, 60, winrows/2-3, wincols/2-30);
-  box(prg, 0, 0);
-  wattron(prg, A_BOLD);
-  mvwaddstr(prg, 0, 4, dat == NULL ? "Calculating..." : "Recalculating...");
-  wattroff(prg, A_BOLD);
+  nccreate(10, 60, dat == NULL ? "Calculating..." : "Recalculating...");
 
-  mvwprintw(prg, 2, 2, "Total items: %-8d size: %s",
+  ncprint(2, 2, "Total items: %-8d size: %s",
     parent->items, cropsize(parent->size));
-  mvwprintw(prg, 3, 2, "Current dir: %s", cropdir(cdir, 43));
-  mvwaddstr(prg, 8, 43, "Press q to quit");
+  ncprint(3, 2, "Current dir: %s", cropdir(cdir, 43));
+  ncaddstr(8, 43, "Press q to quit");
 
  /* show warning if we couldn't open a dir */
   if(lasterr[0] != '\0') {
-     wattron(prg, A_BOLD);
-     mvwaddstr(prg, 5, 2, "Warning:");
-     wattroff(prg, A_BOLD);
-     mvwprintw(prg, 5, 11, "could not open %-32s", cropdir(lasterr, 32));
-     mvwaddstr(prg, 6, 3, "some directory sizes may not be correct");  
+     attron(A_BOLD);
+     ncaddstr(5, 2, "Warning:");
+     attroff(A_BOLD);
+     ncprint(5, 11, "could not open %-32s", cropdir(lasterr, 32));
+     ncaddstr(6, 3, "some directory sizes may not be correct");  
   }
 
  /* animation - but only if the screen refreshes more than or once every second */
@@ -171,29 +168,24 @@ static void drawProgress(char *cdir) {
           ani[i] = antext[i];
   } else
     strcpy(ani, antext);
-  mvwaddstr(prg, 8, 3, ani);
+  ncaddstr(8, 3, ani);
 
-  wrefresh(prg);
-  delwin(prg);
+  refresh();
 }
 
 
 /* show error if can't open parent dir */
 static void drawError(char *dir) {
-  WINDOW *err;
+  nccreate(10, 60, "Error!");
 
-  err = newwin(10, 60, winrows/2-3, wincols/2-30);
-  box(err, 0, 0);
-  wattron(err, A_BOLD);
-  mvwaddstr(err, 0, 4, "Error!");
+  attron(A_BOLD);
+  ncaddstr(5, 2, "Error:");
+  attroff(A_BOLD);
 
-  mvwaddstr(err, 5, 2, "Error:");
-  wattroff(err, A_BOLD);
-  mvwprintw(err, 5, 9, "could not open %s", cropdir(dir, 34));
-  mvwaddstr(err, 6, 3, "press any key to continue...");
+  ncprint(5, 9, "could not open %s", cropdir(dir, 34));
+  ncaddstr(6, 3, "press any key to continue...");
 
-  wrefresh(err);
-  delwin(err);
+  refresh();
 }
 
 
@@ -283,6 +275,16 @@ int calcDir(struct dir *dest, char *path) {
     d->name = malloc(strlen(f)+1);
     strcpy(d->name, f);
 
+#ifdef __CYGWIN__
+   /* /proc/registry names may contain slashes */
+    if(strchr(d->name, '/') || strchr(d->name,  '\\')) {
+      serr = 1;
+      errno = 0;
+      d->flags |= FF_ERR;
+      continue;
+    }
+#endif
+
    /* get full path */
     strcpy(tmp, path);
     strcat(tmp, f);
@@ -315,7 +317,7 @@ int calcDir(struct dir *dest, char *path) {
 
    /* count the size */
     if(!(d->flags & FF_EXL || d->flags & FF_OTHFS)) {
-      d->size = fs.st_blocks * 512;
+      d->size = fs.st_blocks * S_BLKSIZE;
       d->asize = fs.st_size;
       for(t = dest; t != NULL; t = t->parent) {
         t->size += d->size;
@@ -340,17 +342,8 @@ int calcDir(struct dir *dest, char *path) {
   }
 
   if(dest->sub) {
-   /* add reference to parent dir */
-    d = calloc(sizeof(struct dir), 1);
-    d->flags |= FF_PAR;
-    d->name = malloc(3);
-    strcpy(d->name, "..");
-    d->next = dest->sub;
-    d->parent = dest;
-    dest->sub = d;
-
    /* calculate subdirectories */
-    while((d = d->next) != NULL)
+    for(d = dest->sub; d != NULL; d = d->next)
       if(d->flags & FF_DIR && !(d->flags & FF_EXL || d->flags & FF_OTHFS)) {
         strcpy(tmp, path);
         strcat(tmp, d->name);
@@ -372,9 +365,10 @@ struct dir *showCalc(char *path) {
   *lasterr = '\0';
   anpos = 0;
   lastupdate = 999;
+  memset(tmp, 0, PATH_MAX);
 
  /* init parent dir */
-  if(rpath(path, tmp) == NULL || lstat(tmp, &fs) != 0) {
+  if(rpath(path, tmp) == NULL || lstat(tmp, &fs) != 0 || !S_ISDIR(fs.st_mode)) {
     do {
       ncresize();
       if(dat != NULL)
@@ -384,7 +378,7 @@ struct dir *showCalc(char *path) {
     return(NULL);
   }
   parent = calloc(sizeof(struct dir), 1);
-  parent->size = fs.st_blocks * 512;
+  parent->size = fs.st_blocks * S_BLKSIZE;
   parent->asize = fs.st_size;
   parent->flags |= FF_DIR;
   curdev = fs.st_dev;
@@ -396,12 +390,6 @@ struct dir *showCalc(char *path) {
     freedir(parent);
     return(NULL);
   }
- 
- /* remove reference to parent dir if we are in the parent dir */
-  t = parent->sub;
-  parent->sub = t->next;
-  free(t->name);
-  free(t);
 
   return(parent);
 }
