@@ -1,6 +1,6 @@
 /* ncdu - NCurses Disk Usage 
     
-  Copyright (c) 2007 Yoran Heling
+  Copyright (c) 2007-2008 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -29,6 +29,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <stdarg.h>
 #include <ctype.h>
 #include <limits.h>
 #include <errno.h>
@@ -43,6 +44,11 @@
 #include <sys/time.h>
 #include <dirent.h>
 
+/* set S_BLKSIZE if not defined already in sys/stat.h */
+#ifndef S_BLKSIZE
+# define S_BLKSIZE 512
+#endif
+
 /* PATH_MAX 260 on Cygwin is too small for /proc/registry */
 #ifdef __CYGWIN__
 # if PATH_MAX < 1024
@@ -75,6 +81,12 @@
 # define S_ISLNK(x) (x & S_IFLNK)
 #endif
 
+/* check nccreate in util.c for more info on these defines */
+#define ncaddstr(r, c, s) mvaddstr(subwinr+(r), subwinc+(c), s)
+#define  ncaddch(r, c, s)  mvaddch(subwinr+(r), subwinc+(c), s)
+#define   ncmove(r, c)        move(subwinr+(r), subwinc+(c))
+
+
 /*
  *    G L O B A L   F L A G S
  */
@@ -86,7 +98,6 @@
 #define FF_EXL    0x10 /* excluded using exlude patterns */
 #define FF_SERR   0x20 /* error in subdirectory */
 #define FF_BSEL   0x40 /* selected */
-#define FF_PAR    0x80 /* reference to parent directory (hack) */
 
 /* Settings Flags (int sflags) */
 #define SF_SMFS   0x01 /* same filesystem */
@@ -105,14 +116,15 @@
 #define BF_AS     0x40 /* show apparent sizes instead of disk usage */
 
 
+
 /*
  *    S T R U C T U R E S
  */
 struct dir {
   struct dir *parent, *next, *sub;
-  char *name;
+  unsigned char *name;
   off_t size, asize;
-  unsigned int items;
+  unsigned long items;
   unsigned char flags;
 }; 
 
@@ -129,6 +141,8 @@ extern int winrows, wincols;
 /* global settings */
 extern char sdir[PATH_MAX];
 extern int sflags, bflags, sdelay, bgraph;
+/* used for creating windows */
+extern int subwinr, subwinc;
 
 
 /*
@@ -139,11 +153,10 @@ extern char *cropdir(const char *, int);
 extern char *cropsize(const off_t);
 extern char *fullsize(const off_t);
 extern void ncresize(void);
+extern void nccreate(int, int, char *);
+extern void ncprint(int, int, char *, ...);
 extern struct dir * freedir(struct dir *);
 extern char *getpath(struct dir *, char *);
-/* settings.c */
-extern int settingsCli(int, char **);
-extern int settingsWin(void);
 /* calc.c */
 extern struct dir *showCalc(char *);
 /* browser.c */
@@ -157,4 +170,3 @@ extern struct dir *showDelete(struct dir *);
 extern void addExclude(char *);
 extern int addExcludeFile(char *);
 extern int matchExclude(char *);
-
